# -*- coding: utf-8 -*-

# Tests for the molecular structure utilities

#-----------------------------------------------------------------------------
#       Copyright (C) 2014 The Mosaic Development Team
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file LICENSE.txt, distributed as part of this software.
#-----------------------------------------------------------------------------

import unittest

from mosaic.xml_io import XMLReader
import mosaic.mutable_model as M
from mosaic.molecular_structure import FragmentIterator


def peptide_predicate(fragment):
    return fragment.is_polymer and fragment.polymer_type == 'polypeptide'

def two_atoms(fragment):
    return fragment.number_of_atoms == 2

def three_atoms(fragment):
    return fragment.number_of_atoms == 3

class PDB2ONXTest(unittest.TestCase):

    def setUp(self):
        data = dict(XMLReader("2ONX.xml"))
        self.universe = data['universe']

    def test_peptide_chains(self):
        peptide_chains = list(FragmentIterator(self.universe,
                                               peptide_predicate))
        self.assertEqual(len(peptide_chains), 1)
        chain, atom_index, site_index = peptide_chains[0]
        self.assertEqual(chain.label, 'A')
        self.assertEqual(atom_index, 0)
        self.assertEqual(site_index, 0)


class CalphaModelTest(unittest.TestCase):

    def setUp(self):
        water = M.Fragment("water", "water", (),
                           (M.Atom("H1", M.Element("H")),
                            M.Atom("H2", M.Element("H")),
                            M.Atom("O",  M.Element("O"))),
                           (("H1", "O", "single"), ("H2", "O", "single")))
        peptide_chain = M.Polymer('chain', 'ALA,LYS',
                                  (M.Fragment('ALA1', 'ALA', (),
                                              (M.Atom('CA',
                                                      M.CGParticle('CA'), 1),),
                                              ()),
                                   M.Fragment('LYS2', 'LYS', (),
                                              (M.Atom('CA',
                                                      M.CGParticle('CA'), 2),),
                                              ())),
                                  (('ALA1.CA', 'LYS2.CA', ''),),
                                  'polypeptide')
        protein = M.Fragment('protein', 'protein', (peptide_chain,), (), ())
        self.universe = M.Universe('infinite', [(protein, 2), (water, 5)])

    def test_peptide_chains(self):
        peptide_chains = list(FragmentIterator(self.universe,
                                               peptide_predicate))
        self.assertEqual(len(peptide_chains), 2)
        chain, atom_index, site_index = peptide_chains[0]
        self.assertEqual(chain.label, 'chain')
        self.assertEqual(atom_index, 0)
        self.assertEqual(site_index, 0)
        chain, atom_index, site_index = peptide_chains[1]
        self.assertEqual(chain.label, 'chain')
        self.assertEqual(atom_index, 2)
        self.assertEqual(site_index, 3)

    def test_template_peptide_chains(self):
        peptide_chains = list(FragmentIterator(self.universe, peptide_predicate,
                                               template=True))
        self.assertEqual(len(peptide_chains), 1)
        chain, atom_index, site_index = peptide_chains[0]
        self.assertEqual(chain.label, 'chain')
        self.assertEqual(atom_index, 0)
        self.assertEqual(site_index, 0)

    def test_three_atoms(self):
        matches = list(FragmentIterator(self.universe, three_atoms))
        self.assertEqual(len(matches), 5)
        for i, (fragment, atom_index, site_index) in enumerate(matches):
            self.assertEqual(fragment.label, 'water')
            self.assertEqual(atom_index, 4+3*i)
            self.assertEqual(site_index, 6+3*i)

    def test_template_three_atoms(self):
        matches = list(FragmentIterator(self.universe, three_atoms,
                                    template=True))
        self.assertEqual(len(matches), 1)
        fragment, atom_index, site_index = matches[0]
        self.assertEqual(fragment.label, 'water')
        self.assertEqual(atom_index, 2)
        self.assertEqual(site_index, 3)

    def test_two_atoms(self):
        matches = list(FragmentIterator(self.universe, two_atoms))
        self.assertEqual(len(matches), 2)
        for i, (fragment, atom_index, site_index) in enumerate(matches):
            self.assertEqual(fragment.label, 'protein')
            self.assertEqual(atom_index, 2*i)
            self.assertEqual(site_index, 3*i)

    def test_two_atoms_descend(self):
        matches = list(FragmentIterator(self.universe, two_atoms,
                                        descend_on_match=True))
        self.assertEqual(len(matches), 4)
        for i in range(2):
            fragment, atom_index, site_index = matches[2*i]
            self.assertEqual(fragment.label, 'protein')
            self.assertEqual(atom_index, 2*i)
            self.assertEqual(site_index, 3*i)
            fragment, atom_index, site_index = matches[2*i+1]
            self.assertEqual(fragment.label, 'chain')
            self.assertEqual(atom_index, 2*i)
            self.assertEqual(site_index, 3*i)

    def test_template_two_atoms_descend(self):
        matches = list(FragmentIterator(self.universe, two_atoms,
                                        template=True,
                                        descend_on_match=True))
        self.assertEqual(len(matches), 2)
        fragment, atom_index, site_index = matches[0]
        self.assertEqual(fragment.label, 'protein')
        self.assertEqual(atom_index, 0)
        self.assertEqual(site_index, 0)
        fragment, atom_index, site_index = matches[1]
        self.assertEqual(fragment.label, 'chain')
        self.assertEqual(atom_index, 0)
        self.assertEqual(site_index, 0)

def suite():
    loader = unittest.TestLoader()
    s = unittest.TestSuite()
    s.addTest(loader.loadTestsFromTestCase(CalphaModelTest))
    s.addTest(loader.loadTestsFromTestCase(PDB2ONXTest))
    return s

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Tests for the PDB importer

#-----------------------------------------------------------------------------
#       Copyright (C) 2013 The Mosaic Development Team
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file LICENSE.txt, distributed as part of this software.
#-----------------------------------------------------------------------------

import unittest

from mosaic.xml_io import XMLReader
from mosaic_pdb.import_structure import make_models, MMCIFStructure

class PDB2ONXTest(unittest.TestCase):

    def setUp(self):
        self.pdb_model = dict(make_models("2ONX.cif"))
        self.xml_model = dict(XMLReader("2ONX.xml"))

    def test_equivalence(self):
        for name, data in self.pdb_model.items():
            self.assertTrue(name in self.xml_model)
            self.assertTrue(data.is_equivalent(self.xml_model[name]))

class AuthSpecTest(unittest.TestCase):

    def setUp(self):
        self.structure = MMCIFStructure(structure_file="2ONX.cif",
                                        with_auth_spec=True)

    def test_auth_spec(self):
        auth_spec = self.structure.auth_spec
        self.assertFalse(auth_spec is None)
        model = self.structure.models[0]
        for asym_id, sites in model.items():
            for residue in sites:
                for atom in residue:
                    unique_id, atom_id, alt_id, comp_id, seq_id, \
                        ins_code, atom_type, x, y, z, occupancy, \
                        u_iso = atom
                    auth = auth_spec[unique_id]
                    self.assertEqual(comp_id, auth['comp_id'])
                    self.assertEqual(seq_id, auth['seq_id'])
                    self.assertEqual(asym_id, auth['asym_id'])
                    self.assertEqual(atom_id, auth['atom_id'])

def suite():
    loader = unittest.TestLoader()
    s = unittest.TestSuite()
    s.addTest(loader.loadTestsFromTestCase(PDB2ONXTest))
    s.addTest(loader.loadTestsFromTestCase(AuthSpecTest))
    return s

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Molecular structure utilities

.. moduleauthor:: Konrad Hinsen

Utilities for working with hierarchical molecular structures.
"""

#-----------------------------------------------------------------------------
#       Copyright (C) 2014 The Mosaic Development Team
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file LICENSE.txt, distributed as part of this software.
#-----------------------------------------------------------------------------

class FragmentIterator(object):

    """Iterator over fragments that match some predicate

    Usage example for iterating over peptide chains:

        def peptide_predicate(fragment):
            return fragment.is_polymer and fragment.polymer_type == 'polypeptide'

       for chain, atom_index, site_index in FragmentIterator(universe, peptide_predicate):
           ...
    """

    class PseudoFragment(object):

        def __init__(self, universe, template=False):
            if template:
                self.fragments = [f for f, c in universe.molecules]
            else:
                self.fragments = sum([c*[f]
                                      for f, c in universe.molecules],
                                     [])
            self.atoms = ()

        is_polymer = False

    def __init__(self, universe, predicate,
                 descend_on_match=False, template=False):
        """
        :parameter universe: a universe
        :type universe: :class:`mosaic.api.MosaicUniverse`
        :parameter predicate: an object called with the fragment as its only
                              argument and expected to return True or False
        :type predicate: callable
        :parameter descend_on_match: if True, iterate over the subfragments
                                     of fragments that match the predicate
        :type descend_on_match: bool
        :parameter template: if True, iterate over template fragments,
                             otherwise iterate over molecules
        :type template: bool
        """
        self.predicate = predicate
        self.descend_on_match = descend_on_match
        self.stack = [FragmentIterator.PseudoFragment(universe, template)]
        self.atom_index = 0
        self.site_index = 0

    def __iter__(self):
        return self

    def __next__(self):
        while self.stack:
            item = self.stack.pop(0)
            if isinstance(item, tuple):
                a, s = item
                self.atom_index += a
                self.site_index += s
            else:
                match = None
                push = []
                if not isinstance(item, FragmentIterator.PseudoFragment) \
                   and self.predicate(item):
                    match = (item, self.atom_index, self.site_index)
                    if self.descend_on_match:
                        push.extend(item.fragments)
                    else:
                        for f in item.fragments:
                            push.append((f.number_of_atoms, f.number_of_sites))
                else:
                    push.extend(item.fragments)
                push.append((len(item.atoms),
                             sum(a.number_of_sites
                                 for a in item.atoms)))
                for new_item in reversed(push):
                    self.stack.insert(0, new_item)
                if match is not None:
                    return match
        raise StopIteration

    next = __next__
